if not LibStub then return end

local icon = LibStub('LibDBIcon-1.0')
local tip = LibStub('LibQTip-1.0')

-- upvalues
local CalendarCloseEvent = CalendarCloseEvent
local CalendarGetDate = CalendarGetDate
local CalendarGetEventInfo = CalendarGetEventInfo
local CalendarGetNumDayEvents = CalendarGetNumDayEvents
local CalendarOpenEvent = CalendarOpenEvent
local CalendarSetMonth = CalendarSetMonth
local EasyMenu = EasyMenu
local OpenCalendar = OpenCalendar

local addonName, addonTable = ...
local L = addonTable.L

-- cache for events
local todaysEvents = {}
local upcomingEvents = {}
local indicateNew = false
local tooltip = nil

obj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('Broker_Calendar', {
	type = 'data source',
	text = 'Calendar',
	icon = 'Interface\\Icons\\spell_holy_borrowedtime',
})
local obj = obj
local frame = CreateFrame('Frame')
frame:SetScript('OnEvent', function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
frame:RegisterEvent('PLAYER_ENTERING_WORLD')
frame:RegisterEvent('CALENDAR_UPDATE_EVENT_LIST')
frame:RegisterEvent('CALENDAR_UPDATE_INVITE_LIST')


local function ToggleMinimap()
	local hide = not CalendarDB.minimap.hide
	CalendarDB.minimap.hide = hide
	if hide then
		icon:Hide('Broker_Calendar')
	else
		icon:Show('Broker_Calendar')
	end
end

local function printTwoCharNumber(number)
  if number < 10 then
    return '0'..number
  else
    return number
  end
end

local function getStatus(status)
  if status == CALENDAR_INVITESTATUS_INVITED then
    return CALENDAR_STATUS_INVITED
  elseif status == CALENDAR_INVITESTATUS_ACCEPTED then
    return CALENDAR_STATUS_ACCEPTED
  elseif status == CALENDAR_INVITESTATUS_DECLINED then
    return CALENDAR_STATUS_DECLINED
  elseif status == CALENDAR_INVITESTATUS_CONFIRMED then
    return CALENDAR_STATUS_CONFIRMED
  elseif status == CALENDAR_INVITESTATUS_OUT then
    return CALENDAR_STATUS_OUT
  elseif status == CALENDAR_INVITESTATUS_STANDBY then
    return CALENDAR_STATUS_STANDBY
  elseif status == CALENDAR_INVITESTATUS_SIGNEDUP then
    return CALENDAR_STATUS_SIGNEDUP
  elseif status == CALENDAR_INVITESTATUS_NOT_SIGNEDUP then
    return CALENDAR_STATUS_NOT_SIGNEDUP
  elseif status == CALENDAR_INVITESTATUS_TENTATIVE then
    return CALENDAR_STATUS_TENTATIVE
  else
    return ''
  end
end

local function populateEvents(day, month)
  -- today
  local numEvents = CalendarGetNumDayEvents(0, day)
  if numEvents > 0 then
    local title, hour, minute
    for eventIndex = 1, numEvents do
      title, hour, minute, calendarType, _, _, _, _, inviteStatus, invitedBy = CalendarGetDayEvent(0, day, eventIndex)

      if calendarType ~= 'RAID_LOCKOUT' and calendarType ~= 'RAID_RESET' then
        todaysEvents[title] = {hour, minute, invitedBy, getStatus(inviteStatus)}
      end
      
      if inviteStatus == CALENDAR_INVITESTATUS_INVITED and calendarType ~= 'RAID_LOCKOUT' and calendarType ~= 'RAID_RESET' and calendarType ~= 'HOLIDAY' then
        indicateNew = true
      end
    end
  end

  -- upcoming
  local numEvents, title, days
  local monthOffset = 0
  local daysInMonth = select(3, CalendarGetMonth())
  for dayOffset = 1, 7 do
    if day + dayOffset > daysInMonth then
      monthOffset = monthOffset + 1
      day = 1 - dayOffset
      daysInMonth = select(3, CalendarGetMonth(monthOffset))
    end

    numEvents = CalendarGetNumDayEvents(monthOffset, day + dayOffset)
    if numEvents ~= 0 then
      for eventIndex = 1, numEvents do
        title, hour, minute, calendarType, _, _, _, _, inviteStatus, invitedBy  = CalendarGetDayEvent(monthOffset, day + dayOffset, eventIndex)

        if calendarType == 'PLAYER' or (upcomingEvents[title] == nil and todaysEvents[title] == nil and calendarType ~= 'RAID_LOCKOUT' and calendarType ~= 'RAID_RESET') then
          upcomingEvents[title] = {day + dayOffset, month + monthOffset, invitedBy, getStatus(inviteStatus)}
        end
      
        if inviteStatus == CALENDAR_INVITESTATUS_INVITED and calendarType ~= 'RAID_LOCKOUT' and calendarType ~= 'RAID_RESET' and calendarType ~= 'HOLIDAY' then
          indicateNew = true
        end
      end
    end
  end
  
  CalendarCloseEvent()

  local sortedEvents = {}
  for title, time in pairs(todaysEvents) do
    table.insert(sortedEvents, {title = title, hour = time[1], minute = time[2], by = time[3], status = time[4]})
  end

  if #sortedEvents > 1 then
    -- sort by start time
    table.sort(sortedEvents, function(a, b)
      if a.hour == nil or b.hour == nil then
        return
      elseif a.hour == b.hour then
        if a.minute == b.minute then
          return a.title < b.title
        else
          return a.minute < b.minute
        end
      else
        return a.hour < b.hour
      end
    end)
  end

  todaysEvents = sortedEvents

  sortedEvents = {}
  for title, start in pairs(upcomingEvents) do
    table.insert(sortedEvents, {title = title, day = start[1], month = start[2], by = start[3], status = start[4]})
  end

  if #sortedEvents > 1 then
    -- sort by start time
    table.sort(sortedEvents, function(a, b)
      if a.month == nil or b.month == nil then
        return
      elseif a.month == b.month then
        if a.day == b.day then
          return a.title < b.title
        else
          return a.day < b.day
        end
      else
        return a.month < b.month
      end
    end)
  end

  upcomingEvents = sortedEvents
  sortedEvents = nil
  
  if indicateNew then
    obj.text = L["NEW_EVENT"]
  else
    obj.text = 'Calendar'
  end
end

local function reloadEvents()
  OpenCalendar()
  todaysEvents = nil
  upcomingEvents = nil
  indicateNew = false
  todaysEvents = {}
  upcomingEvents = {}

  local _, month, day = CalendarGetDate()
  populateEvents(day, month)
end

local function color(text, colorCode)
  local color
  if colorCode == 'y' then
    color = '|cffffd100'
  elseif colorCode == 'o' then
    color = '|cffe59933'
  end
  
  return color..text..'|r'
end

local function showTodaysEvents()
  if #todaysEvents == 0 then
    tooltip:AddLine(L['NO_EVENTS_TODAY'])
  else
    tooltip:AddLine(
      color(L['TODAY']..':', 'y'),
      '',
      color(L['STARTS'], 'y')
    )

    for i = 1, #todaysEvents do
      if todaysEvents[i].hour ~= nil then
        tooltip:AddLine(
          todaysEvents[i].title..(todaysEvents[i].by == '' and '' or ' ('..L['BY']..' '..todaysEvents[i].by..')')..(todaysEvents[i].by == '' and '' or color(' - '..todaysEvents[i].status, 'o')),
          '',
          printTwoCharNumber(todaysEvents[i].hour)..':'..printTwoCharNumber(todaysEvents[i].minute)
        )
      end
    end
  end
end

local function showUpcomingEvents()
  if #upcomingEvents > 0 then
    tooltip:AddLine(' ')
    tooltip:AddLine(
      color(L['COMMING_SOON']..':', 'y'),
      '',
      color(L['STARTS'], 'y')
    )

    for i = 1, #upcomingEvents do
      if upcomingEvents[i].day ~= nil then
        tooltip:AddLine(
          upcomingEvents[i].title..(upcomingEvents[i].by == '' and '' or ' ('..L['BY']..' '..upcomingEvents[i].by..')')..(upcomingEvents[i].by == '' and '' or color(' - '..upcomingEvents[i].status, 'o')),
          '',
          printTwoCharNumber(upcomingEvents[i].day)..'.'..printTwoCharNumber(upcomingEvents[i].month)..'.'
        )
      end
    end
  end
end

-- All credit for this func goes to Tekkub and his picoGuild!
local function GetTipAnchor(frame)
  local x, y = frame:GetCenter()
  if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end
  local hhalf = (x > UIParent:GetWidth() * 2 / 3) and 'RIGHT' or (x < UIParent:GetWidth() / 3) and 'LEFT' or ''
  local vhalf = (y > UIParent:GetHeight() / 2) and 'TOP' or 'BOTTOM'
  return vhalf..hhalf, frame, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end

local function toggleMinimap()
  local hide = not CalendarDB.minimap.hide
	CalendarDB.minimap.hide = hide
	if hide then
		icon:Hide('Broker_Calendar')
	else
		icon:Show('Broker_Calendar')
	end
end

local function getMenu()
  local menu = {}
  local entry = {}
  -- title
  entry.text = 'Broker Calendar - '..L['OPTIONS']
  entry.isTitle = true
  menu[1] = entry
  
  -- separator
  entry = {}
  entry.text = ''
  entry.disabled = true
  entry.separator = true
  menu[2] = entry
  
  -- show on minimap
  entry = {}
  entry.text = L['ATT_MINIMAP']
  entry.checked = not CalendarDB.minimap.hide
  entry.func = toggleMinimap
  menu[3] = entry
  
  return menu
end

local function getMenuFrame(self)
  frame:SetPoint(GetTipAnchor(self))
  frame.displayMode = 'MENU'
  frame.relativeTo = self
  return frame
end

function frame:PLAYER_ENTERING_WORLD()
	-- PortalsDB.minimap is there for smooth upgrade of SVs from old version
	if not CalendarDB then
		CalendarDB = {}
		CalendarDB.minimap = {}
		CalendarDB.minimap.hide = false
		CalendarDB.version = 1
	end

	if icon then
		icon:Register('Broker_Calendar', obj, CalendarDB.minimap)
	end

	self:UnregisterEvent('PLAYER_ENTERING_WORLD')

  local _, month, day, year = CalendarGetDate()
  obj.text = printTwoCharNumber(day)..'. '..printTwoCharNumber(month)..'. '..printTwoCharNumber(year)
  OpenCalendar()
  CalendarSetMonth(0)
  populateEvents(day, month)
end

function frame:CALENDAR_UPDATE_EVENT_LIST()
  reloadEvents()
end
function frame:CALENDAR_UPDATE_INVITE_LIST()
  reloadEvents()
end

function obj.OnClick(self, button)
  GameTooltip:Hide()
  if button == 'RightButton' then
    EasyMenu(getMenu(), getMenuFrame(self))
  else
    GameTimeFrame_OnClick(GameTimeFrame)
  end
end

function obj.OnLeave()
	tip:Release(tooltip)
end

function obj.OnEnter(self)
  if #todaysEvents == 0 and #upcomingEvents == 0 then
    reloadEvents()
  end

  tooltip = tip:Acquire('BrokerCalendarTooltip', 4, 'LEFT', 'RIGHT', 'RIGHT')
  local weekday, month, day, year = CalendarGetDate()

  tooltip:AddHeader(
    color('Calendar', 'y'),
    '',
    color(printTwoCharNumber(day)..'. '..printTwoCharNumber(month)..'. '..printTwoCharNumber(year), 'y'),
    ''
  )
  tooltip:AddLine(' ')

  showTodaysEvents()
  showUpcomingEvents()

  tooltip:SmartAnchorTo(self)
  tooltip:Show()
end

-- slashcommand definition
SlashCmdList['BROKER_CALENDAR'] = function() ToggleMinimap() end
SLASH_BROKER_CALENDAR1 =  '/b_calendar'